@@ -1,12 +1,11 @@
 #include <stddef.h>
-#include "src/util/c99_stdint.h"
+#include <stdint.h>
 #include <set>
 #include <string>
-#include <valarray>
 #include <vector>
 
 #include "src/adfa/adfa.h"
-#include "src/codegen/code.h"
+#include "src/codegen/output.h"
 #include "src/codegen/helpers.h"
 #include "src/options/opt.h"
 #include "src/encoding/enc.h"
@@ -16,14 +15,12 @@
 #include "src/util/file_utils.h"
 #include "src/util/string_utils.h"
 
-
 namespace re2c {
 
-static void from_le(Output &output, CodeList *code, const char *expr, size_t size)
-{
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    const char *text;
+static void from_le(Output& output, CodeList* code, const char* expr, size_t size) {
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    const char* text;
 
     append(code, code_text(alc, "/* from little-endian to host-endian */"));
     text = o.cstr("unsigned char *p = (unsigned char*)&").cstr(expr).flush();
@@ -36,12 +33,11 @@ static void from_le(Output &output, CodeList *code, const char *expr, size_t siz
     append(code, code_stmt(alc, text));
 }
 
-Code *emit_skeleton_prolog(Output &output)
-{
-    code_alc_t &alc = output.allocator;
-    const char *if_cond;
-    CodeList *code, *block, *if_code;
-    CodeArgs *args;
+Code* emit_skeleton_prolog(Output& output) {
+    OutAllocator& alc = output.allocator;
+    const char* if_cond;
+    CodeList* code, *block, *if_code;
+    CodeArgs* args;
 
     code = code_list(alc);
 
@@ -70,7 +66,7 @@ Code *emit_skeleton_prolog(Output &output)
     append(block, code_stmt(alc, "FILE *f = fopen(fname, \"rb\")"));
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "goto error"));
-    append(block, code_if_then_else(alc, "f == NULL", if_code, NULL, false));
+    append(block, code_if_then_else(alc, "f == NULL", if_code, nullptr, false));
 
     append(block, code_newline(alc));
 
@@ -85,7 +81,7 @@ Code *emit_skeleton_prolog(Output &output)
     append(block, code_stmt(alc, "buffer = malloc(unit * (fsize + padding))"));
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "goto error"));
-    append(block, code_if_then_else(alc, "buffer == NULL", if_code, NULL, false));
+    append(block, code_if_then_else(alc, "buffer == NULL", if_code, nullptr, false));
 
     append(block, code_newline(alc));
 
@@ -93,7 +89,7 @@ Code *emit_skeleton_prolog(Output &output)
     if_cond = "fread(buffer, unit, fsize, f) != fsize";
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "goto error"));
-    append(block, code_if_then_else(alc, if_cond, if_code, NULL, false));
+    append(block, code_if_then_else(alc, if_cond, if_code, nullptr, false));
 
     append(block, code_newline(alc));
 
@@ -104,26 +100,24 @@ Code *emit_skeleton_prolog(Output &output)
     append(block, code_newline(alc));
     append(block, code_slabel(alc, "error"));
 
-    append(block, code_stmt(alc,
-        "fprintf(stderr, \"error: cannot read file '%s'\\n\", fname)"));
+    append(block, code_stmt(alc, "fprintf(stderr, \"error: cannot read file '%s'\\n\", fname)"));
     append(block, code_stmt(alc, "free(buffer)"));
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "fclose(f)"));
-    append(block, code_if_then_else(alc, "f != NULL", if_code, NULL, false));
+    append(block, code_if_then_else(alc, "f != NULL", if_code, nullptr, false));
     append(block, code_stmt(alc, "return NULL"));
 
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
-    return code_block(alc, code, CodeBlock::RAW);
+    return code_block(alc, code, CodeBlock::Kind::RAW);
 }
 
-static void emit_skeleton_defines(Output &output, CodeList *code, const DFA &dfa)
-{
-    const opt_t *opts = output.block().opts;
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    const size_t sizeof_cunit = opts->encoding.szCodeUnit();
-    const char *text;
+static void emit_skeleton_defines(Output& output, CodeList* code, const Adfa& dfa) {
+    const opt_t* opts = output.block().opts;
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    const size_t sizeof_cunit = opts->encoding.cunit_size();
+    const char* text;
 
     append(code, code_newline(alc));
     text = o.cstr("#define YYCTYPE ").exact_uint(sizeof_cunit).flush();
@@ -151,20 +145,19 @@ static void emit_skeleton_defines(Output &output, CodeList *code, const DFA &dfa
     }
     append(code, code_textraw(alc, "#define YYSHIFT(o) cursor += o"));
     append(code, code_textraw(alc, "#define YYLESSTHAN(n) (limit - cursor) < n"));
-    append(code, code_textraw(alc, opts->eof == NOEOF
-        ? "#define YYFILL(n) { goto loop_end; }"
-        : "#define YYFILL(n) 1 /* fail */"));
+    append(code, code_textraw(alc, opts->fill_eof == NOEOF
+            ? "#define YYFILL(n) { goto loop_end; }"
+            : "#define YYFILL(n) 1 /* fail */"));
     append(code, code_newline(alc));
 }
 
-static void emit_skeleton_function_action(Output &output, CodeList *code, const DFA &dfa)
-{
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
+static void emit_skeleton_function_action(Output& output, CodeList* code, const Adfa& dfa) {
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
     const uint64_t norule = rule2key(Rule::NONE, dfa.key_size, dfa.def_rule);
-    CodeArgs *args;
-    CodeList *if_code, *else_code, *body;
-    const char *if_cond, *text;
+    CodeArgs* args;
+    CodeList* if_code, *else_code, *body;
+    const char* if_cond, *text;
 
     args = code_args(alc);
     append(args, code_arg(alc, "unsigned *pkix"));
@@ -189,13 +182,12 @@ static void emit_skeleton_function_action(Output &output, CodeList *code, const
     if_code = code_list(alc);
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"warning: lex_").str(dfa.name)
-        .cstr(": control flow is undefined\"\n")
-        .cstr("\" for input at position %ld, rerun re2c with '-W'\\n\"").flush();
+    text = o.cstr("\"warning: lex_").str(dfa.name).cstr(": control flow is undefined\"\n")
+            .cstr("\" for input at position %ld, rerun re2c with '-W'\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "pos"));
     append(if_code, code_fcall(alc, "fprintf", args, ";"));
-    append(body, code_if_then_else(alc, if_cond, if_code, NULL));
+    append(body, code_if_then_else(alc, if_cond, if_code, nullptr));
 
     if_cond = "len_act == len_exp && rule_act == rule_exp";
     if_code = code_list(alc);
@@ -205,10 +197,9 @@ static void emit_skeleton_function_action(Output &output, CodeList *code, const
     else_code = code_list(alc);
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"error: lex_").str(dfa.name)
-        .cstr(": at position %ld (key %u):\\n\"\n")
-        .cstr("\"\\texpected: match length %ld, rule %u\\n\"\n")
-        .cstr("\"\\tactual:   match length %ld, rule %u\\n\"").flush();
+    text = o.cstr("\"error: lex_").str(dfa.name).cstr(": at position %ld (key %u):\\n\"\n")
+            .cstr("\"\\texpected: match length %ld, rule %u\\n\"\n")
+            .cstr("\"\\tactual:   match length %ld, rule %u\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "pos"));
     append(args, code_arg(alc, "kix"));
@@ -220,19 +211,18 @@ static void emit_skeleton_function_action(Output &output, CodeList *code, const
     append(else_code, code_text(alc, "return 1;"));
     append(body, code_if_then_else(alc, if_cond, if_code, else_code));
 
-    append(code, code_block(alc, body, CodeBlock::WRAPPED));
+    append(code, code_block(alc, body, CodeBlock::Kind::WRAPPED));
 }
 
-static void emit_skeleton_stags(Output &output, CodeList *code, const DFA &dfa)
-{
-    const opt_t *opts = output.block().opts;
-    if (dfa.stagvars.empty() && !opts->posix_syntax) return;
+static void emit_skeleton_stags(Output& output, CodeList* code, const Adfa& dfa) {
+    const opt_t* opts = output.block().opts;
+    if (dfa.stagvars.empty() && !opts->tags_posix_syntax) return;
 
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    CodeArgs *args;
-    CodeList *if_code, *body;
-    const char *if_cond, *text;
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    CodeArgs* args;
+    CodeList* if_code, *body;
+    const char* if_cond, *text;
 
     append(code, code_newline(alc));
 
@@ -259,15 +249,14 @@ static void emit_skeleton_stags(Output &output, CodeList *code, const DFA &dfa)
     if_cond = "exp == act || (exp == NIL && tag == NULL)";
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "return 0"));
-    append(body, code_if_then_else(alc, if_cond, if_code, NULL));
+    append(body, code_if_then_else(alc, if_cond, if_code, nullptr));
 
     append(body, code_newline(alc));
 
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"error: lex_").str(dfa.name)
-        .cstr(": at position %ld, key %u: \"\n")
-        .cstr("\"wrong value for tag '%s': expected %u, actual %u\\n\"").flush();
+    text = o.cstr("\"error: lex_").str(dfa.name).cstr(": at position %ld, key %u: \"\n")
+            .cstr("\"wrong value for tag '%s': expected %u, actual %u\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "token - input"));
     append(args, code_arg(alc, "kix"));
@@ -277,18 +266,17 @@ static void emit_skeleton_stags(Output &output, CodeList *code, const DFA &dfa)
     append(body, code_fcall(alc, "fprintf", args, ";"));
     append(body, code_stmt(alc, "return 1"));
 
-    append(code, code_block(alc, body, CodeBlock::WRAPPED));
+    append(code, code_block(alc, body, CodeBlock::Kind::WRAPPED));
 }
 
-static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
-{
-    code_alc_t &alc = output.allocator;
-    CodeArgs *args;
-    CodeList *if_code, *block;
-    const char *if_cond;
+static void emit_skeleton_mtag_defs(Output& output, CodeList* code) {
+    OutAllocator& alc = output.allocator;
+    CodeArgs* args;
+    CodeList* if_code, *block;
+    const char* if_cond;
 
-    // Use `void *` instead of `YYCTYPE *`, as YYCTYPE may vary for different
-    // blocks. and this definition must be valid throughout the whole file.
+    // Use `void *` instead of `YYCTYPE *`, as YYCTYPE may vary for different blocks. and this
+    // definition must be valid throughout the whole file.
 
     append(code, code_newline(alc));
 
@@ -296,7 +284,7 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     block = code_list(alc);
     append(block, code_text(alc, "ptrdiff_t pred;"));
     append(block, code_text(alc, "const void *elem;"));
-    append(code, code_block(alc, block, CodeBlock::INDENTED));
+    append(code, code_block(alc, block, CodeBlock::Kind::INDENTED));
     append(code, code_text(alc, "} yymtag_t;"));
 
     append(code, code_newline(alc));
@@ -306,7 +294,7 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     append(block, code_text(alc, "yymtag_t *head;"));
     append(block, code_text(alc, "yymtag_t *next;"));
     append(block, code_text(alc, "yymtag_t *last;"));
-    append(code, code_block(alc, block, CodeBlock::INDENTED));
+    append(code, code_block(alc, block, CodeBlock::Kind::INDENTED));
     append(code, code_text(alc, "} yymtagpool_t;"));
 
     append(code, code_newline(alc));
@@ -316,7 +304,7 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     append(code, code_fdecl(alc, "static void yymtagpool_clear", args));
     block = code_list(alc);
     append(block, code_stmt(alc, "tp->next = tp->head"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
     append(code, code_newline(alc));
 
@@ -325,11 +313,10 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     append(code, code_fdecl(alc, "static void yymtagpool_init", args));
     block = code_list(alc);
     append(block, code_stmt(alc, "static const unsigned size = 4096"));
-    append(block, code_stmt(alc,
-        "tp->head = (yymtag_t*)malloc(size * sizeof(yymtag_t))"));
+    append(block, code_stmt(alc, "tp->head = (yymtag_t*)malloc(size * sizeof(yymtag_t))"));
     append(block, code_stmt(alc, "tp->next = tp->head"));
     append(block, code_stmt(alc, "tp->last = tp->head + size"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
     append(code, code_newline(alc));
 
@@ -339,7 +326,7 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     block = code_list(alc);
     append(block, code_stmt(alc, "free(tp->head)"));
     append(block, code_stmt(alc, "tp->head = tp->next = tp->last = NULL"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
     append(code, code_newline(alc));
 
@@ -350,16 +337,16 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     if_cond = "tp->next == tp->last";
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "const unsigned size = tp->last - tp->head"));
-    append(if_code, code_stmt(alc,
-        "yymtag_t *head = (yymtag_t*)malloc(2 * size * sizeof(yymtag_t))"));
+    append(if_code, code_stmt(
+            alc, "yymtag_t *head = (yymtag_t*)malloc(2 * size * sizeof(yymtag_t))"));
     append(if_code, code_stmt(alc, "memcpy(head, tp->head, size * sizeof(yymtag_t))"));
     append(if_code, code_stmt(alc, "free(tp->head)"));
     append(if_code, code_stmt(alc, "tp->head = head"));
     append(if_code, code_stmt(alc, "tp->next = head + size"));
     append(if_code, code_stmt(alc, "tp->last = head + size * 2"));
-    append(block, code_if_then_else(alc, if_cond, if_code, NULL));
+    append(block, code_if_then_else(alc, if_cond, if_code, nullptr));
     append(block, code_stmt(alc, "return tp->next++"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
     append(code, code_newline(alc));
 
@@ -373,24 +360,23 @@ static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
     append(block, code_stmt(alc, "n->pred = *pt"));
     append(block, code_stmt(alc, "n->elem = t"));
     append(block, code_stmt(alc, "*pt = n - tp->head"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 
     append(code, code_newline(alc));
 }
 
-static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
-{
+static void emit_skeleton_mtags(Output& output, CodeList* code, const Adfa& dfa) {
     if (dfa.mtagvars.empty()) return;
 
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    CodeArgs *args;
-    CodeList *if_code, *block, *body;
-    const char *if_cond, *text;
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    CodeArgs* args;
+    CodeList* if_code, *block, *body;
+    const char* if_cond, *text;
 
-    // Data structures for m-tags are the same for all blocks in the file. They
-    // are emitted at the first block that has m-tags, because it's impossible
-    // to know earlier (when generating prologue) if m-tags are needed at all.
+    // Data structures for m-tags are the same for all blocks in the file. They are emitted at the
+    // first block that has m-tags, because it's impossible to know earlier (when generating
+    // prologue) if m-tags are needed at all.
     if (!output.done_mtag_defs) {
         output.done_mtag_defs = true;
         emit_skeleton_mtag_defs(output, code);
@@ -420,16 +406,15 @@ static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
     if_code = code_list(alc);
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"error: lex_").str(dfa.name)
-        .cstr(": at position %ld, key %u: \"\n")
-        .cstr("\"history for tag '%s' is too short\\n\"").flush();
+    text = o.cstr("\"error: lex_").str(dfa.name).cstr(": at position %ld, key %u: \"\n")
+            .cstr("\"history for tag '%s' is too short\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "token - input"));
     append(args, code_arg(alc, "kix + n"));
     append(args, code_arg(alc, "name"));
     append(if_code, code_fcall(alc, "fprintf", args, ";"));
     append(if_code, code_stmt(alc, "return 1"));
-    append(block, code_if_then_else(alc, "mtag == -1", if_code, NULL));
+    append(block, code_if_then_else(alc, "mtag == -1", if_code, nullptr));
 
     append(block, code_stmt(alc, "const YYCTYPE *tag = (tp->head + mtag)->elem"));
     append(block, code_stmt(alc, "mtag = (tp->head + mtag)->pred"));
@@ -441,9 +426,8 @@ static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
     if_code = code_list(alc);
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"error: lex_").str(dfa.name)
-        .cstr(": at position %ld, key %u: \"\n")
-        .cstr("\"wrong value for tag '%s': expected %u, actual %u\\n\"").flush();
+    text = o.cstr("\"error: lex_").str(dfa.name).cstr(": at position %ld, key %u: \"\n")
+            .cstr("\"wrong value for tag '%s': expected %u, actual %u\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "token - input"));
     append(args, code_arg(alc, "kix + n"));
@@ -452,37 +436,34 @@ static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
     append(args, code_arg(alc, "act"));
     append(if_code, code_fcall(alc, "fprintf", args, ";"));
     append(if_code, code_stmt(alc, "return 1"));
-    append(block, code_if_then_else(alc, if_cond, if_code, NULL));
+    append(block, code_if_then_else(alc, if_cond, if_code, nullptr));
 
-    append(body, code_block(alc, block, CodeBlock::INDENTED));
+    append(body, code_block(alc, block, CodeBlock::Kind::INDENTED));
     append(body, code_text(alc, "}")); // end of for loop
 
     if_code = code_list(alc);
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
-    text = o.cstr("\"error: lex_").str(dfa.name)
-        .cstr(": at position %ld, key %u: \"\n")
-        .cstr("\"history for tag '%s' is too long\\n\"").flush();
+    text = o.cstr("\"error: lex_").str(dfa.name).cstr(": at position %ld, key %u: \"\n")
+            .cstr("\"history for tag '%s' is too long\\n\"").flush();
     append(args, code_arg(alc, text));
     append(args, code_arg(alc, "token - input"));
     append(args, code_arg(alc, "kix + n"));
     append(args, code_arg(alc, "name"));
     append(if_code, code_fcall(alc, "fprintf", args, ";"));
     append(if_code, code_stmt(alc, "return 1"));
-    append(body, code_if_then_else(alc, "mtag != -1", if_code, NULL));
+    append(body, code_if_then_else(alc, "mtag != -1", if_code, nullptr));
     append(body, code_text(alc, "return 0;"));
 
-    append(code, code_block(alc, body, CodeBlock::WRAPPED));
+    append(code, code_block(alc, body, CodeBlock::Kind::WRAPPED));
 }
 
-static void emit_skeleton_function_check_key_count(Output &output, CodeList *code,
-    DFA &dfa)
-{
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    CodeArgs *args;
-    CodeList *block, *if_code;
-    const char *if_cond, *text;
+static void emit_skeleton_function_check_key_count(Output& output, CodeList* code, Adfa& dfa) {
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    CodeArgs* args;
+    CodeList* block, *if_code;
+    const char* if_cond, *text;
 
     append(code, code_newline(alc));
 
@@ -496,26 +477,25 @@ static void emit_skeleton_function_check_key_count(Output &output, CodeList *cod
     if_cond = "used + need <= have";
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "return 0"));
-    append(block, code_if_then_else(alc, if_cond, if_code, NULL));
+    append(block, code_if_then_else(alc, if_cond, if_code, nullptr));
     args = code_args(alc);
     append(args, code_arg(alc, "stderr"));
     text = o.cstr("\"error: lex_").str(dfa.name).cstr(": not enough keys\\n\"").flush();
     append(args, code_arg(alc, text));
     append(block, code_fcall(alc, "fprintf", args, ";"));
     append(block, code_stmt(alc, "return 1"));
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 }
 
-static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
-{
-    const opt_t *opts = output.block().opts;
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    const size_t sizeof_cunit = opts->encoding.szCodeUnit();
+static void emit_skeleton_function_lex(Output& output, CodeList* code, Adfa& dfa) {
+    const opt_t* opts = output.block().opts;
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    const size_t sizeof_cunit = opts->encoding.cunit_size();
     std::set<std::string>::const_iterator var1, var2;
-    CodeArgs *args;
-    CodeList *block, *block2, *if_code, *if1_code;
-    const char *text;
+    CodeArgs* args;
+    CodeList* block, *block2, *if_code, *if1_code;
+    const char* text;
 
     std::string filename = opts->output_file;
     if (filename.empty()) {
@@ -528,8 +508,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     text = o.cstr("int lex_").str(dfa.name).cstr("()").flush();
     append(code, code_text(alc, text));
     block = code_list(alc);
-    text = o.cstr("const size_t padding = ").u64(dfa.max_fill)
-        .cstr("; /* YYMAXFILL */").flush();
+    text = o.cstr("const size_t padding = ").u64(dfa.max_fill).cstr("; /* YYMAXFILL */").flush();
     append(block, code_text(alc, text));
     append(block, code_stmt(alc, "int status = 0"));
     append(block, code_stmt(alc, "size_t input_len = 0"));
@@ -540,10 +519,9 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     append(block, code_stmt(alc, "const YYCTYPE *limit = NULL"));
     append(block, code_stmt(alc, "const YYCTYPE *token = NULL"));
     append(block, code_stmt(alc, "const YYCTYPE *eof = NULL"));
-    if (opts->posix_syntax) {
+    if (opts->tags_posix_syntax) {
         append(block, code_stmt(alc, "size_t yynmatch"));
-        text = o.cstr("const YYCTYPE *yypmatch[").u64(dfa.max_nmatch)
-            .cstr(" * 2]").flush();
+        text = o.cstr("const YYCTYPE *yypmatch[").u64(dfa.max_nmatch).cstr(" * 2]").flush();
         append(block, code_stmt(alc, text));
     }
     append(block, code_stmt(alc, "unsigned int i = 0"));
@@ -566,7 +544,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "status = 1"));
     append(if_code, code_stmt(alc, "goto end"));
-    append(block, code_if_then_else(alc, "input == NULL", if_code, NULL));
+    append(block, code_if_then_else(alc, "input == NULL", if_code, nullptr));
 
     append(block, code_newline(alc));
 
@@ -574,8 +552,8 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
         text = o.cstr("for (i = 0; i < input_len; ++i) {").flush();
         append(block, code_text(alc, text));
         block2 = code_list(alc);
-        from_le(output, block2, "input[i]", opts->encoding.szCodeUnit());
-        append(block, code_block(alc, block2, CodeBlock::INDENTED));
+        from_le(output, block2, "input[i]", opts->encoding.cunit_size());
+        append(block, code_block(alc, block2, CodeBlock::Kind::INDENTED));
         append(block, code_text(alc, "}"));
         append(block, code_newline(alc));
     }
@@ -590,7 +568,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     if_code = code_list(alc);
     append(if_code, code_stmt(alc, "status = 1"));
     append(if_code, code_stmt(alc, "goto end"));
-    append(block, code_if_then_else(alc, "keys == NULL", if_code, NULL));
+    append(block, code_if_then_else(alc, "keys == NULL", if_code, nullptr));
 
     append(block, code_textraw(alc, ""));
 
@@ -599,7 +577,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
         append(block, code_text(alc, text));
         block2 = code_list(alc);
         from_le(output, block2, "keys[i]", dfa.key_size);
-        append(block, code_block(alc, block2, CodeBlock::INDENTED));
+        append(block, code_block(alc, block2, CodeBlock::Kind::INDENTED));
         append(block, code_text(alc, "}"));
         append(block, code_textraw(alc, ""));
     }
@@ -610,16 +588,16 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
 
     append(block, code_textraw(alc, ""));
 
-    // Don't use `for` loops or `while` loops: the generated code may contain an
-    // inner loop if the --loop-switch option is used, and then `continue` and
-    // `break` statements may result in unexpected control flow. Use `goto` loop.
+    // Don't use `for` loops or `while` loops: the generated code may contain an inner loop if the
+    // --loop-switch option is used, and then `continue` and `break` statements may result in
+    // unexpected control flow. Use `goto` loop.
     append(block, code_stmt(alc, "i = 0"));
     append(block, code_slabel(alc, "loop"));
 
     block2 = code_list(alc);
     append(block2, code_stmt(alc, "goto loop_end"));
-    append(block, code_if_then_else(alc,
-        "!(status == 0 && cursor < eof && i < keys_count)", block2, NULL));
+    append(block, code_if_then_else(
+            alc, "!(status == 0 && cursor < eof && i < keys_count)", block2, nullptr));
 
     block2 = code_list(alc);
     append(block2, code_stmt(alc, "token = cursor"));
@@ -638,9 +616,9 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     }
     // autogenerated stag variables
     if (!dfa.stagnames.empty()) {
-        text = o.cstr("\n").str(indent(2, opts->indString))
-            .cstr("const YYCTYPE *@@ = NULL;").flush();
-        Code *stags = code_fmt(alc, CODE_STAGS, NULL, text, NULL);
+        text = o.cstr("\n").str(indent(2, opts->indent_str)).cstr("const YYCTYPE *@@ = NULL;")
+                .flush();
+        Code* stags = code_fmt(alc, CodeKind::STAGS, nullptr, text, nullptr);
         gen_tags(o, opts, stags, dfa.stagnames);
         append(block2, stags);
         append(block2, code_textraw(alc, ""));
@@ -662,9 +640,8 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     if (!dfa.mtagnames.empty()) {
         append(block2, code_text(alc, "yymtagpool_clear(&yytp);"));
 
-        text = o.cstr("\n").str(indent(2, opts->indString))
-            .cstr("ptrdiff_t @@ = -1;").flush();
-        Code *mtags = code_fmt(alc, CODE_MTAGS, NULL, text, NULL);
+        text = o.cstr("\n").str(indent(2, opts->indent_str)).cstr("ptrdiff_t @@ = -1;").flush();
+        Code* mtags = code_fmt(alc, CodeKind::MTAGS, nullptr, text, nullptr);
         gen_tags(o, opts, mtags, dfa.mtagnames);
         append(block2, mtags);
         append(block2, code_textraw(alc, ""));
@@ -683,24 +660,24 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
         }
     }
 
-    if (opts->bFlag) {
-        append(block2, gen_bitmap(output, dfa.bitmap));
+    if (opts->bitmaps) {
+        append(block2, gen_bitmap(output, dfa.bitmap, dfa.cond));
     }
     append(block2, code_textraw(alc, ""));
 
     if (!opts->loop_switch) {
         gen_dfa_as_blocks_with_labels(output, dfa, block2);
     } else {
-        // With loop/switch there are no labels, and each block has its own state switch.
-        // Restart state counter from zero so that cases start from zero.
+        // With loop/switch there are no labels, and each block has its own state switch. Restart
+        // state counter from zero so that cases start from zero.
         output.label_counter = 0;
-        CodeCases *cases = code_cases(alc);
+        CodeCases* cases = code_cases(alc);
         gen_dfa_as_switch_cases(output, dfa, cases);
         wrap_dfas_in_loop_switch(output, block2, cases);
     }
     append(block2, code_textraw(alc, ""));
 
-    append(block, code_block(alc, block2, CodeBlock::WRAPPED));
+    append(block, code_block(alc, block2, CodeBlock::Kind::WRAPPED));
     append(block, code_slabel(alc, "loop_end"));
 
     if_code = code_list(alc);
@@ -709,18 +686,18 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     append(if1_code, code_stmt(alc, "status = 1"));
     append(if1_code, code_stmt(alc, "const long pos = token - input"));
     text = o.cstr("fprintf(stderr, \"error: lex_").str(dfa.name)
-        .cstr(": unused input strings left at position %ld\\n\", pos)").flush();
+            .cstr(": unused input strings left at position %ld\\n\", pos)").flush();
     append(if1_code, code_stmt(alc, text));
-    append(if_code, code_if_then_else(alc, "cursor != eof", if1_code, NULL, false));
+    append(if_code, code_if_then_else(alc, "cursor != eof", if1_code, nullptr, false));
 
     if1_code = code_list(alc);
     append(if1_code, code_stmt(alc, "status = 1"));
     text = o.cstr("fprintf(stderr, \"error: lex_").str(dfa.name)
-        .cstr(": unused keys left after %u keys\\n\", i)").flush();
+            .cstr(": unused keys left after %u keys\\n\", i)").flush();
     append(if1_code, code_stmt(alc, text));
-    append(if_code, code_if_then_else(alc, "i != keys_count", if1_code, NULL, false));
+    append(if_code, code_if_then_else(alc, "i != keys_count", if1_code, nullptr, false));
 
-    append(block, code_if_then_else(alc, "status == 0", if_code, NULL, false));
+    append(block, code_if_then_else(alc, "status == 0", if_code, nullptr, false));
 
     append(block, code_textraw(alc, ""));
     append(block, code_slabel(alc, "end"));
@@ -732,13 +709,12 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     append(block, code_textraw(alc, ""));
     append(block, code_stmt(alc, "return status"));
 
-    append(code, code_block(alc, block, CodeBlock::WRAPPED));
+    append(code, code_block(alc, block, CodeBlock::Kind::WRAPPED));
 }
 
-static void emit_skeleton_undefs(Output &output, CodeList *code, DFA &dfa)
-{
-    const opt_t *opts = output.block().opts;
-    code_alc_t &alc = output.allocator;
+static void emit_skeleton_undefs(Output& output, CodeList* code, Adfa& dfa) {
+    const opt_t* opts = output.block().opts;
+    OutAllocator& alc = output.allocator;
 
     append(code, code_textraw(alc, ""));
     append(code, code_textraw(alc, "#undef YYCTYPE"));
@@ -762,8 +738,7 @@ static void emit_skeleton_undefs(Output &output, CodeList *code, DFA &dfa)
     append(code, code_textraw(alc, "#undef YYFILL"));
 }
 
-void emit_skeleton(Output &output, CodeList *code, DFA &dfa)
-{
+void emit_skeleton(Output& output, CodeList* code, Adfa& dfa) {
     emit_skeleton_defines(output, code, dfa);
     emit_skeleton_function_action(output, code, dfa);
     emit_skeleton_stags(output, code, dfa);
@@ -773,46 +748,43 @@ void emit_skeleton(Output &output, CodeList *code, DFA &dfa)
     emit_skeleton_undefs(output, code, dfa);
 }
 
-Code *emit_skeleton_epilog(Output &output)
-{
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
+Code* emit_skeleton_epilog(Output& output) {
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
 
-    CodeList *stmts = code_list(alc);
-    const std::set<std::string> &xs = output.skeletons;
-    for (std::set<std::string>::const_iterator i = xs.begin(); i != xs.end(); ++i) {
-        const char *if_cond = o.cstr("lex_").str(*i).cstr("() != 0").flush();
-        CodeList *if_code = code_list(alc);
+    CodeList* stmts = code_list(alc);
+    for (const std::string& s : output.skeletons) {
+        const char* if_cond = o.cstr("lex_").str(s).cstr("() != 0").flush();
+        CodeList* if_code = code_list(alc);
         append(if_code, code_stmt(alc, "return 1"));
-        append(stmts, code_if_then_else(alc, if_cond, if_code, NULL, false));
+        append(stmts, code_if_then_else(alc, if_cond, if_code, nullptr, false));
     }
     append(stmts, code_stmt(alc, "return 0"));
 
-    CodeList *main = code_list(alc);
+    CodeList* main = code_list(alc);
     append(main, code_text(alc, ""));
     append(main, code_text(alc, "int main()"));
-    append(main, code_block(alc, stmts, CodeBlock::WRAPPED));
+    append(main, code_block(alc, stmts, CodeBlock::Kind::WRAPPED));
 
-    return code_block(alc, main, CodeBlock::RAW);
+    return code_block(alc, main, CodeBlock::Kind::RAW);
 }
 
-void emit_skeleton_action(Output &output, CodeList *code, const DFA &dfa, size_t rid)
-{
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    const std::string &name = dfa.name;
-    const Rule &r = dfa.rules[rid];
+void emit_skeleton_action(Output& output, CodeList* code, const Adfa& dfa, size_t rid) {
+    OutAllocator& alc = output.allocator;
+    Scratchbuf& o = output.scratchbuf;
+    const std::string& name = dfa.name;
+    const Rule& r = dfa.rules[rid];
     const uint64_t rkey = rule2key(rid, dfa.key_size, dfa.def_rule);
-    const char *text;
+    const char* text;
     std::vector<std::string> fintags;
 
     size_t ntag = 3;
     for (size_t t = r.ltag; t < r.htag; ++t) {
-        const Tag &tag = dfa.tags[t];
+        const Tag& tag = dfa.tags[t];
         if (t != r.ttag && !fictive(tag)) ++ntag;
     }
 
-    CodeArgs *args = code_args(alc);
+    CodeArgs* args = code_args(alc);
     append(args, code_arg(alc, "keys_count"));
     append(args, code_arg(alc, "i"));
     text = o.u64(ntag).flush();
@@ -820,7 +792,7 @@ void emit_skeleton_action(Output &output, CodeList *code, const DFA &dfa, size_t
     text = o.cstr("status = check_key_count_").str(dfa.name).flush();
     append(code, code_fcall(alc, text, args, ""));
 
-    CodeList *hangafter = code_list(alc);
+    CodeList* hangafter = code_list(alc);
 
     args = code_args(alc);
     append(args, code_arg(alc, "&i"));
@@ -834,7 +806,7 @@ void emit_skeleton_action(Output &output, CodeList *code, const DFA &dfa, size_t
     append(hangafter, code_fcall(alc, text, args, ntag > 3 ? "" : ";"));
 
     for (size_t t = r.ltag; t < r.htag; ++t) {
-        const Tag &tag = dfa.tags[t];
+        const Tag& tag = dfa.tags[t];
         if (t == r.ttag || fictive(tag)) continue;
 
         const bool mtag = history(tag);
@@ -856,7 +828,7 @@ void emit_skeleton_action(Output &output, CodeList *code, const DFA &dfa, size_t
         append(hangafter, code_fcall(alc, text, args, ntag > 3 ? "" : ";"));
     }
 
-    append(code, code_block(alc, hangafter, CodeBlock::INDENTED));
+    append(code, code_block(alc, hangafter, CodeBlock::Kind::INDENTED));
 
     append(code, code_stmt(alc, "goto loop"));
 }
